# --------------------------------------------------------------
#  HTMLDocument --
# 
#       HTML document class.
#       
# 
#  Results:
#       XXX
# 
#  Side effects:
#       XXX
# 
# --------------------------------------------------------------

itcl_class HTMLDocument {
    constructor {config} {}
    destructor {}
    method configure {config} {}

    method write {f} {
	puts $f "Content-type: text/html"
	puts $f ""
	foreach i $components {
	    $i write $f
	}
    }
    method += {e} {
	lappend components $e
    }
    
    public title {}
    public components {}
}


# --------------------------------------------------------------
#  HTML components --
# 
#       HTML component types, lists, headers, links, etc.
#       
# 
#  Results:
#       XXX
# 
#  Side effects:
#       XXX
# 
# --------------------------------------------------------------
itcl_class HTMLcomponent {
}

itcl_class HTMLlist {
    inherit HTMLcomponent
    constructor {config} {}
    destructor {}
    method write {f} {
	puts $f "<UL>"
	foreach i $elems {
	    puts "<LI>"
	    $i write $f
	}
	puts $f "</UL>"
    }
    # takes HTMLcomponent types
    method += {e} {
	lappend elems $e
    }
    
    public elems {}
}

itcl_class HTMLtxt {
    inherit HTMLcomponent
    constructor {config} {}
    destructor {}
    method write {f} {
	puts $f $txt
    }
    public txt {}
}

itcl_class HTMLcomponents {
    inherit HTMLcomponent
    constructor {config} {}
    destructor {}
    # takes HTMLcomponent types
    method += {e} {
	lappend components $e
    }
    method write {f} {
	foreach i $components {
	    $i write $f
	}
    }
    public components {}
}

itcl_class HTMLbold {
    inherit HTMLcomponent
    constructor {config} {}
    destructor {}
    # takes HTMLcomponent types
    method += {e} {
	lappend components $e
    }
    method write {f} {
	puts $f "<B>"
	foreach i $components {
	    $i write $f
	}
	puts $f "</B>"
    }
    public components {}
}

itcl_class HTMLpre {
    inherit HTMLcomponent
    constructor {config} {}
    destructor {}

    method write {f} {
	puts $f "<PRE>$txt</PRE>"
    }
    public txt {}
}

itcl_class HTMLheader {
    inherit HTMLcomponent
    constructor {config} {}
    destructor {}
    method write {f} {
	puts $f "<H$level>"
	$txt write $f
	puts $f "</H$level>"
    }
    public txt {}
    public level {1}
}

itcl_class HTMLparagraph {
    inherit HTMLcomponent
    constructor {config} {}
    destructor {}
    method write {f} {
	puts $f "$txt<P>"
    }
    public txt {}
}

itcl_class HTMLnamedsection {
    inherit HTMLcomponent
    constructor {config} {}
    destructor {}
    method write {f} {
	puts $f "<A name=\"$name\">"
	foreach i $components {
	    $i write $f
	}
	puts $f "</A>"
    }
    # takes HTMLcomponent types
    method += {e} {
	lappend components $e
    }
    
    public name {}
    public components {}
}

itcl_class HTMLlink {
    inherit HTMLcomponent
    constructor {config} {}
    destructor {}
    method write {f} {
	puts $f "<A HREF=\"$link\">"
	$txt write $f
	puts $f "</A>"
    }

    public txt {}
    public link {}
}

itcl_class HTMLdesclistItem {
    inherit HTMLcomponent
    constructor {config} {}
    destructor {}
    method write {f} {
	puts $f "<DT>"
	$title write $f
	puts $f "<DD>"
	if {$description!=""} {
	    $description write $f
	}
    }
    public title {}
    public description {}
}

itcl_class HTMLdesclist {
    inherit HTMLcomponent
    constructor {config} {}
    destructor {}
    method write {f} {
	puts $f "<DL>"
	foreach i $items {
	    $i write $f
	}
	puts $f "</DL>"
    }

    method += {e} {
	lappend items $e
    }
    public items {}
}

itcl_class HTMLform {
    inherit HTMLcomponent
    constructor {config} {}
    destructor {}
    method write {f} {
	puts "<form action=\"$action\">"
	foreach i $components {
	    $i write $f
	}
	puts "</form>"
    }
    method += {e} {
	lappend components $e
    }
    public action {}
    public components {}
}

itcl_class HTMLformInput {
    inherit HTMLcomponent
    constructor {config} {}
    destructor {}
    method write {f} {
	if {$size!=""} {
	    set sz "size=$size"
	} else {
	    set sz ""
	}
	puts "<input name=\"$varname\" value=\"$defvalue\" $sz>"
    }

    public varname {}
    public defvalue {}
    public size {}
}

itcl_class HTMLformSubmit {
    inherit HTMLcomponent
    constructor {config} {}
    destructor {}
    method write {f} {
	puts "<input type=\"submit\" value=\"$txt\">"
    }
    public txt {}
}

itcl_class HTMLformReset {
    inherit HTMLcomponent
    constructor {config} {}
    destructor {}
    method write {f} {
	puts "<input type=\"reset\" value=\"$txt\">"
    }
    public txt {}
}

itcl_class HTMLformOption {
    inherit HTMLcomponent
    constructor {config} {}
    destructor {}
    method configure {config} {}

    method write {f} {
	if $multiple {
	    set mul MULTIPLE
	} else {
	    set mul ""
	}
	puts $f "<select name=\"$varname\" $mul>"
	foreach i $values {
	    set sel ""
	    foreach j $defvalues {
		if {$i==$j} {
		    set sel SELECTED
		} else {
		    set sel ""
		}
	    }

	    # we don't do $i write $f because values can't be formatted.
	    puts $f "<option $sel> $i"
	}
	puts $f "</select>"
    }

    # 1 == pull-down menu.  > 1 == listbox 
    public size {1}

    # Multiple selection possible
    public multiple {0}
    public varname {}
    public values {}
    public defvalues {}
}

# composite of descriptive list of italicized items and radio inputs.
# Should actually be built out of individual components...

itcl_class HTMLformRadio {
    inherit HTMLcomponent
    constructor {config} {}
    destructor {}
    method write {f} {
	puts $f "<DL>"
	loop i 0 [llength $values] {
	    if {$defvalue == [lindex $values $i]} {
		set checked CHECKED
	    } else {
		set checked ""
	    }
	    puts $f "<DD> <INPUT TYPE=\"radio\" NAME=\"$varname\" VALUE=\"[lindex $values $i]\" $checked><I>[lindex $descriptions $i]</I>"
	}
	puts $f "</DL>"
    }
    
    public varname {}
    public values {}
    public defvalue {}
    public descriptions {}
}

itcl_class HTMLformCheck {
    inherit HTMLcomponent
    constructor {config} {}
    destructor {}
    method write {f} {
	if {$defvalue=="on"} {
	    set val checked
	} else {
	    set val ""
	}
	puts $f "<input type=\"checkbox\" name=\"$varname\" $val>"
    }
    
    public varname {}
    public defvalue {off}
}

itcl_class HTMLformText {
    inherit HTMLcomponent
    constructor {config} {}
    destructor {}
    method write {f} {
	puts "<textarea name=\"$varname\" rows=$rows cols=$cols>$defvalue</textarea>"
    }
    public rows 4
    public cols 60
    public varname {}
    public defvalue {}
}

# separator (currently <hr>)
itcl_class HTMLsep {
    inherit HTMLcomponent
    constructor {} {}
    destructor {}
    method write {f} {
	puts $f "<HR>"
    }
}
